__version__ = '0.9.1'


def description(message):
    pass


def _description(message):
    pass


def it(message):
    pass


def _it(message):
    pass


def context(message):
    pass


def _context(message):
    pass


def before():
    pass


def after():
    pass
